@@ -31,14 +31,9 @@
 #include <linux/kdev_t.h>	/* for MKDEV */
 #include <linux/major.h>	/* for LOOP_MAJOR */
 #endif
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#include <locale.h>
-#define _(String) gettext (String)
-#else
-#define _(String) (String)
-#endif
+#include <errno.h>
 
+#include "i18n.h"
 #include "comm.h"
 #include "loop.h"         /* for loop_info */
 #include "signals.h"
@@ -516,24 +511,19 @@ scan_mounts (void)
     {
       if (sscanf (line, "%as %as", &path, &mounted) != 2)
 	continue;
-      /* new kernel :-) */
-      if (stat (path, &st_dev) < 0) {
-	free (path);            /* might be NFS or such */
-	free (mounted);
-	continue;		
-      }
-      if (S_ISBLK (st_dev.st_mode) && MAJOR (st_dev.st_rdev) == LOOP_MAJOR)
-	{
-          struct loop_info loopinfo;
-          int fd;
+      if ( stat (path, &st_dev) == 0) {
+      	if (S_ISBLK (st_dev.st_mode) && MAJOR (st_dev.st_rdev) == LOOP_MAJOR) {
+          	struct loop_info loopinfo;
+        	int fd;
 
-          if ((fd = open(path, O_RDWR)) > 0) {
-            if (ioctl(fd, LOOP_GET_STATUS, &loopinfo) >= 0) {
-              add_other(it_loop,loopinfo.lo_device,loopinfo.lo_device,loopinfo.lo_inode,path);
-            }
-            (void) close(fd);
+          	if ((fd = open(path, O_RDWR)) > 0) {
+            		if (ioctl(fd, LOOP_GET_STATUS, &loopinfo) >= 0) {
+              			add_other(it_loop,loopinfo.lo_device,loopinfo.lo_device,loopinfo.lo_inode,path);
+            		}
+            		(void) close(fd);
                 }
 	}
+      }
       if (stat (mounted, &st_mounted) < 0)
 	{
 	  perror (mounted);
@@ -590,25 +580,34 @@ scan_swaps (void)
 static int
 ask (pid_t pid)
 {
-  int ch, c;
+  int res;
+  size_t len;
+  char *line;
+
+  line = NULL;
+  len = 0;
 
   fflush (stdout);
   do {
     fprintf (stderr, _("Kill process %d ? (y/N) "), pid);
     fflush (stderr);
-    do {
-      if ((ch = getchar ()) == EOF)
-        exit (0);
-      if (ch == '\n') return 0;
-    } while (ch == '\t' || ch == ' ');
-    do
-      if ((c = getchar ()) == EOF)
-	    exit (0);
-    while (c != '\n');
-  } while (ch != '\n' && ch != 'y' && ch != 'n' && ch != 'Y' && ch != 'N');
-  return ch == 'y' || ch == 'Y';
-}
 
+    if (getline (&line, &len, stdin) < 0)
+      return 0;
+    /* Check for default */
+    if (line[0] == '\n') {
+      free(line);
+      return 0;
+    }
+    res = rpmatch(line);
+    if (res >= 0) {
+      free(line);
+      return res;
+    }
+  } while(1);
+  /* Never should get here */
+  return 0;
+}
 
 static void
 kill_item (const FILE_DSC * file, const ITEM_DSC * item)
@@ -659,7 +658,8 @@ show_files_or_kill (void)
       {
 	if (header && (file->flags & FLAG_VERB))
 	  {
-	    fprintf (stderr, "\n%*s USER        PID ACCESS COMMAND\n", NAME_FIELD, "");
+	    fprintf (stderr, _("\n%*s USER        PID ACCESS COMMAND\n"),
+		     NAME_FIELD, "");
 	    header = 0;
 	  }
 	length = 0;
@@ -964,7 +964,7 @@ usage (void)
     "       fuser -V\n\n"
     "    -a        display unused files too\n"
     "    -c        mounted FS\n"
-    "    -f        silently ignored (for POSIX compatibility\n"
+    "    -f        silently ignored (for POSIX compatibility)\n"
     "    -k        kill processes accessing that file\n"
     "    -i        ask before killing (ignored without -k)\n"
     "    -l        list signal names\n"
@@ -986,10 +986,11 @@ void print_version()
 {
   fprintf(stderr, _("fuser (psmisc) %s\n"), VERSION);
   fprintf(stderr, _(
-    "Copyright (C) 1993-2002 Werner Almesberger and Craig Small\n\n"
+    "Copyright (C) 1993-2002 Werner Almesberger and Craig Small\n\n"));
+  fprintf(stderr, _(
     "PSmisc comes with ABSOLUTELY NO WARRANTY.\n"
-    "This is free software, and you are welcome to redistribute it under the terms\n"
-    "of the GNU General Public License.\n"
+    "This is free software, and you are welcome to redistribute it under\n"
+    "the terms of the GNU General Public License.\n"
     "For more information about these matters, see the files named COPYING.\n"));
 }
 
@@ -1177,7 +1178,7 @@ main (int argc, char **argv)
 	      if (!parse_inet (*argv, this_name_space->name, &lcl_port,
 			       &rmt_addr, &rmt_port))
 		{
-		  fprintf (stderr, _("%s/%s: invalid specificiation\n"), *argv,
+		  fprintf (stderr, _("%s/%s: invalid specification\n"), *argv,
 			   this_name_space->name);
 		  continue;
 		}
