@@ -15,16 +15,15 @@
 #include <getopt.h>
 #include <pwd.h>
 #include <dirent.h>
+#include <curses.h>
+#include <term.h>
 #include <termios.h>
-#include <termcap.h>
 #include <langinfo.h>
-#include <locale.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <libintl.h>
-#define _(String) gettext (String)
 
+#include "i18n.h"
 #include "comm.h"
 
 #ifdef FLASK_LINUX
@@ -124,7 +123,7 @@ out_char (char c)
   cur_x += (c & 0xc0) != 0x80;	/* only count first UTF-8 char */
   if (cur_x <= output_width || !trunc)
     putchar (c);
-  if (cur_x == output_width + 1 && trunc)
+  if (cur_x == output_width + 1 && trunc && ((c & 0xc0) != 0x80))
   {
     if (last_char || (c & 0x80))
       putchar ('+');
@@ -710,37 +709,43 @@ read_stdin (void)
 static void
 usage (void)
 {
-  fprintf (stderr, _("usage: pstree [ -a ] [ -c ] [ -h | -H pid ] [ -l ] [ -n ] [ -p ] [ -u ]\n"));
-  fprintf (stderr, _("              [ -G | -U ] [ pid | user]\n"));
-  fprintf (stderr, _("       pstree -V\n\n"));
-  fprintf (stderr, _("    -a     show command line arguments\n"));
-  fprintf (stderr, _("    -c     don't compact identical subtrees\n"));
-  fprintf (stderr, _("    -h     highlight current process and its ancestors\n"));
-  fprintf (stderr, _("    -H pid highlight process \"pid\" and its ancestors\n"));
-  fprintf (stderr, _("    -G     use VT100 line drawing characters\n"));
-  fprintf (stderr, _("    -l     don't truncate long lines\n"));
-  fprintf (stderr, _("    -n     sort output by PID\n"));
-  fprintf (stderr, _("    -p     show PIDs; implies -c\n"));
-  fprintf (stderr, _("    -u     show uid transitions\n"));
+  fprintf (stderr, _(
+    "usage: pstree [ -a ] [ -c ] [ -h | -H pid ] [ -l ] [ -n ] [ -p ] [ -u ]\n"
+    "              [ -A | -G | -U ] [ pid | user]\n"
+    "       pstree -V\n\n"
+    "    -a     show command line arguments\n"
+    "    -A     use ASCII line drawing characters\n"
+    "    -c     don't compact identical subtrees\n"
+    "    -h     highlight current process and its ancestors\n"
+    "    -H pid highlight process \"pid\" and its ancestors\n"
+    "    -G     use VT100 line drawing characters\n"
+    "    -l     don't truncate long lines\n"
+    "    -n     sort output by PID\n"
+    "    -p     show PIDs; implies -c\n"
+    "    -u     show uid transitions\n"));
 #ifdef FLASK_LINUX
-  fprintf (stderr, _("    -s     show Flask SIDs\n"));
-  fprintf (stderr, _("    -x     show Flask security contexts\n"));
+  fprintf (stderr, _(
+    "    -s     show Flask SIDs\n"
+    "    -x     show Flask security contexts\n"));
 #endif /*FLASK_LINUX*/
-  fprintf (stderr, _("    -U     use UTF-8 (Unicode)) line drawing characters\n"));
-  fprintf (stderr, _("    -V     display version information\n"));
-  fprintf (stderr, _("    pid    start at pid, default 1 (init))\n"));
-  fprintf (stderr, _("    user   show only trees rooted at processes of that user\n\n"));
+  fprintf (stderr, _(
+    "    -U     use UTF-8 (Unicode) line drawing characters\n"
+    "    -V     display version information\n"
+    "    pid    start at pid, default 1 (init)\n"
+    "    user   show only trees rooted at processes of that user\n\n"));
   exit (1);
 }
 
 void print_version()
 {
   fprintf(stderr, _("pstree (psmisc) %s\n"), VERSION);
-  fprintf(stderr, _("Copyright (C) 1993-2002 Werner Almesberger and Craig Small\n\n"));
-  fprintf(stderr, _("pstree comes with ABSOLUTELY NO WARRANTY.\n"));
-  fprintf(stderr, _("This is free software, and you are welcome to redistribute it under the terms\n"));
-  fprintf(stderr, _("of the GNU General Public License.\n"));
-  fprintf(stderr, _("For more information about these matters, see the files named COPYING.\n"));
+  fprintf(stderr, _(
+    "Copyright (C) 1993-2002 Werner Almesberger and Craig Small\n\n"));
+  fprintf(stderr, _(
+    "PSmisc comes with ABSOLUTELY NO WARRANTY.\n"
+    "This is free software, and you are welcome to redistribute it under\n"
+    "the terms of the GNU General Public License.\n"
+    "For more information about these matters, see the files named COPYING.\n"));
 }
 
 
@@ -752,6 +757,7 @@ main (int argc, char **argv)
   const struct passwd *pw;
   pid_t pid, highlight;
   char termcap_area[1024];
+  char *termname;
   int c;
   char *tmpstr;
 
@@ -762,7 +768,11 @@ main (int argc, char **argv)
   highlight = 0;
   pw = NULL;
   
+#ifdef ENABLE_NLS
   setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
+  textdomain(PACKAGE);
+#endif
   
   if ( (tmpstr = strrchr(argv[0],'/'))) {
     tmpstr++;
@@ -770,23 +780,45 @@ main (int argc, char **argv)
       wait_end=1;
   }
 
+  /*
+   * Attempt to figure out a good default symbol set.  Will be overriden by
+   * command-line options, if given.
+   */
+
+  if (!strcmp(nl_langinfo(CODESET), "UTF-8")) {
+    /* Use UTF-8 symbols if the locale's character set is UTF-8. */
+    sym = &sym_utf;
+  } else if ((termname = getenv ("TERM")) && \
+             (strlen (termname) > 0) && \
+             (setupterm (NULL, 1 /* stdout */, NULL) == OK) && \
+             (tigetstr ("acsc") > 0)) {
+    /*
+     * Failing that, if TERM is defined, a non-null value, and the terminal
+     * has the VT100 graphics charset, use it.
+     */
+    sym = &sym_vt100;
+  } else {
+    /* Otherwise, fall back to ASCII. */
+    sym = &sym_ascii;
+  }
 
 #ifdef FLASK_LINUX
-  while ((c = getopt (argc, argv, "acGhH:npluUVsx")) != EOF)
+  while ((c = getopt (argc, argv, "aAcGhH:npluUVsx")) != EOF)
 #else  /*FLASK_LINUX*/
-  while ((c = getopt (argc, argv, "acGhH:npluUV")) != EOF)
+  while ((c = getopt (argc, argv, "aAcGhH:npluUV")) != EOF)
 #endif /*FLASK_LINUX*/
     switch (c)
       {
       case 'a':
 	print_args = 1;
 	break;
+      case 'A':
+	sym = &sym_ascii;
+	break;
       case 'c':
 	compact = 0;
 	break;
       case 'G':
-	if (sym != &sym_ascii)
-	  usage ();
 	sym = &sym_vt100;
 	break;
       case 'h':
@@ -825,8 +857,6 @@ main (int argc, char **argv)
 	user_change = 1;
 	break;
       case 'U':
-	if (sym != &sym_ascii)
-	  usage ();
 	sym = &sym_utf;
 	break;
       case 'V':
@@ -857,14 +887,6 @@ main (int argc, char **argv)
   }
   if (optind != argc)
     usage ();
-  if (sym == &sym_ascii) {
-  	/*
-  	 * If the locale's charset is UTF-8, automatically
-  	 * use the UTF-8 symbols
-  	 */
-	if (!strcmp(nl_langinfo(CODESET), "UTF-8"))
-		sym = &sym_utf;
-  }
   read_proc ();
   for (current = find_proc (highlight); current; current = current->parent)
     current->highlight = 1;
