@@ -8,6 +8,8 @@
 #include <string.h>
 #include <ctype.h>
 #include <signal.h>
+
+#include "i18n.h"
 #include "signals.h"
 
 
@@ -60,6 +62,6 @@ get_signal (char *name, const char *cmd)
       break;
   if (walk->name)
     return walk->number;
-  fprintf (stderr, "%s: unknown signal; %s -l lists signals.\n", name, cmd);
+  fprintf (stderr, _("%s: unknown signal; %s -l lists signals.\n"), name, cmd);
   exit (1);
 }
